/*
 * Tablero.cpp
 *
 *  Created on: 23/06/2011
 *      Author: antonio
 */

#include <iostream>
#include "Tablero.h"

using namespace STDP;


Tablero::Tablero(StdPijo& stdp, Nivel& nivel, Color colorBordesTablero, Color colorLineasTablero, Color colorCursor, char cursor) {
	this->stdp = &stdp;
	this->nivel = &nivel;
	this->colorCursor = colorCursor;
	this->colorBordesTablero = colorBordesTablero;
	this->colorLineasTablero = colorLineasTablero;
	this->cursor = cursor;

	this->margenIzq=2;
	this->margenSup=1;
}

Tablero::~Tablero() {
	// TODO Auto-generated destructor stub
}

void Tablero::imprimirJugador()
{
	stdp->setColor(ABLINK, colorCursor, CBLACK);
	stdp->setCursor(Jugador.x, Jugador.y);
	stdp->writeChar(this->cursor);
}

void Tablero::imprimirMinas()
{
	for(int i=margenSup; i<=nivel->getFilas(); i++){
		for(int j=margenIzq+1; j<nivel->getColumnas(); j+=2){
			if(tablero[i][j].getMina()){
				stdp->setColor(ABLINK, CRED, CBLACK);
				stdp->setCursor(j,i);
				stdp->writeStr("+");
			}
		}
	}
}

void Tablero::imprimirTablero(bool rastro)
{

	for(int i=margenSup; i<=nivel->getFilas(); i++){
		//Pintar columna izquierda
		stdp->setColor(ABLINK, colorBordesTablero, CBLACK);
		stdp->setCursor(margenIzq,i);
		stdp->writeStr("|");
		for(int j=margenIzq+1; j<nivel->getColumnas(); j+=2){
			if(i==margenSup || i==nivel->getFilas()){
				//Pintar filas superior e inferior
				stdp->setColor(ABLINK, colorBordesTablero, CBLACK);
				stdp->setCursor(j,i);
				stdp->writeStr("_|");
			}else if(j==nivel->getColumnas()-1 && i<nivel->getFilas()){
				//Pinta columna derecha
				stdp->setColor(ABLINK, colorLineasTablero, CBLACK);
				stdp->setCursor(j,i);
				stdp->writeStr("_");
				stdp->setColor(ABLINK, colorBordesTablero, CBLACK);
				stdp->setCursor(j+1,i);
				stdp->writeStr("|");
			}else{
				//Pinta una casilla interior
				stdp->setColor(ABLINK, colorLineasTablero, CBLACK);
				stdp->setCursor(j,i);
				stdp->writeStr("_|");
			}
			if(tablero[i][j].getPisada()){
				if(rastro){
					stdp->setColor(ABLINK, colorBordesTablero, CBLACK);
					stdp->setCursor(j,i);
					stdp->writeChar('*');
				}
			}
		}
	}
	//Pinto la línea de puntos superior para cerrar el tablero
	stdp->setColor(ABLINK, colorBordesTablero, CBLACK);
	for(int i=margenIzq+1; i<nivel->getColumnas(); i+=2){
		stdp->setCursor(i,margenSup-1);
		stdp->writeStr("_ ");
	}
	//Pinto el rastro del jugador

	//Posicionamos el jugador en la pantalla
	imprimirJugador();

}

void Tablero::sembrarMinas()
{
	this->Jugador.x = margenIzq+1;
	this->Jugador.y = nivel->getFilas();
//	Vaciamos el tablero
//	También borro el rastro del jugador. (Aunque no sea elegante hacerlo aquí)
	for(int i=0; i<MAXFILA; i++){
		for(int j=0; j<MAXCOLU; j++){
			tablero[i][j].setMina(false);
			tablero[i][j].setPisada(false);
		}
	}


	srand (time(NULL));

	int n = 0;
	int x=0;
	int y=0;
	while(n<nivel->getNumMinas()){
		x = 1;
		n = 0;
		do
			x = rand() % ((nivel->getColumnas())-(margenIzq)) + (margenIzq);
		while(x%2 == 0);
		y = rand() % ((nivel->getFilas()-1)-(margenSup)) + (margenSup+1);

		if(x>0 && x<MAXCOLU && y>0 && y<MAXFILA)
			tablero[y][x].setMina(true);
		//Tengo que hacer este pequeño artificio con tal de obtener el número adecuado
		//de minas dado que los pseudoaleatorias se repiten a menudo.
		for(int i=margenSup-1; i<nivel->getFilas()+1; i++){
			for(int j=margenIzq+1; j<nivel->getColumnas(); j+=2){
				if(tablero[i][j].getMina())
					n++;
			}
		}
	}




}

int Tablero::detectarMinas(Posicion pos)
{
	int minas=0;
	if(tablero[Jugador.y+1][Jugador.x].getMina())
		minas++;
	if(tablero[Jugador.y-1][Jugador.x].getMina())
		minas++;
	if(tablero[Jugador.y+1][Jugador.x+2].getMina())
		minas++;
	if(tablero[Jugador.y+1][Jugador.x-2].getMina())
		minas++;
	if(tablero[Jugador.y-1][Jugador.x+2].getMina())
		minas++;
	if(tablero[Jugador.y-1][Jugador.x-2].getMina())
		minas++;
	if(tablero[Jugador.y][Jugador.x+2].getMina())
		minas++;
	if(tablero[Jugador.y][Jugador.x-2].getMina())
		minas++;
	return minas;
}

bool Tablero::moverJugador(Posicion pos)
{
	//Guardamos la posición anterior
	if(pos.y < nivel->getFilas())
		tablero[pos.y][pos.x].setPisada(true);
	//Determinamos los bordes del tablero y corregimos si es necesario.
	if(pos.x < 2)
		pos.x+=2;
	if(pos.x > nivel->getColumnas())
		pos.x-=2;
	if(pos.y < 1)
		pos.y++;
	if(pos.y > nivel->getFilas())
		pos.y--;
	Jugador = pos;

	//Determinamos si la casilla contiene una mina
	if(tablero[Jugador.y][Jugador.x].getMina())
		return false;
	return true;
}


